#include "LCD.h"
#include "delay.h"

void strobe() {
	LCD_E = 1;
	Nop();
	LCD_E = 0;
	Nop();
}

void send_nibble(byte b) {
	LCD_PORT = (LCD_PORT & 0x0F) | (b & 0xF0);
	strobe();
}

void send_byte(byte b) {
	send_nibble(b); //Send high nibble
	send_nibble(b << 4); //Send low nibble
}

void send_cmd(byte b) {
	LCD_RS = 0; //Command mode
	send_byte(b);
	delay5ms();
}

void send_data(byte b) {
	LCD_RS = 1; //Data mode
	send_byte(b);
	delay100us();
}

void set_cursor(byte row, byte col) {
	byte command = 0x80;
	switch(row) {
	case 1:
		command += col - 1;
		break;
	case 2:
		command += 0x40 + col - 1;
		break;
	case 3:
		command += 0x10 + col - 1;
		break;
	case 4:
		command += 0x50 + col - 1;
		break;
	default:
		command += col - 1;
		break;
	}
	send_cmd(command);
}

void cursor_on() {
	send_cmd(0x0E);
}

void cursor_off() {
	send_cmd(0x0C);
}

void cls() {
	send_cmd(0x01);
}

void sc_r() {
	send_cmd(0x1E);
}

void sc_l() {
	send_cmd(0x18);
}

void home() {
	send_cmd(0x02);
}

byte b2hc(byte data) {	
	switch(data & 0x0F) {
	case 0:
		return '0';
	case 1:
		return '1';
	case 2:
		return '2';
	case 3:
		return '3';
	case 4:
		return '4';
	case 5:
		return '5';
	case 6:
		return '6';
	case 7:
		return '7';
	case 8:
		return '8';
	case 9:
		return '9';
	case 10:
		return 'A';
	case 11:
		return 'B';
	case 12:
		return 'C';
	case 13:
		return 'D';
	case 14:
		return 'E';
	case 15:
		return 'F';
	}
	return 0;
}


void lcd_init() {
	delay100us(); //Delay for LCD to start itself up
	
	LCD_E = 0;
	LCD_RW = 0;
	LCD_RS = 0;
	
	//Start initializing the LCD
	send_nibble(0x30);
	delay5ms();
	send_nibble(0x30);
	delay5ms();	
	send_nibble(0x30);
	delay100us();
	send_nibble(0x20); //4 bit interface
	delay100us();
	
	send_cmd(0b00101000); //Function set: 4 bit interface
	send_cmd(0b00001110); //Display on/off: display on, cursor on, blink off
	send_cmd(0b00000110); //Entry mode: increment, no shift
	send_cmd(0b00000010); //Cursor home
	send_cmd(0b00000001); //Clear DDRAM/LCD
	delay5ms();	
}